﻿//------------------------------------------------------------------------------
// <copyright file="MacroCommand.cs" company="Company">
//     Copyright (c) Company.  All rights reserved.
// </copyright>
//------------------------------------------------------------------------------

using System;
using System.ComponentModel.Design;
using System.Globalization;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using EnvDTE80;
using EnvDTE;

namespace VSMacro
{
    /// <summary>
    /// Command handler
    /// </summary>
    internal sealed class MacroCommand
    {
        /// <summary>
        /// Command ID.
        /// </summary>
        public const int CommandId = 0x3001;

        /// <summary>
        /// Command menu group (command set GUID).
        /// </summary>
        public static readonly Guid CommandSet = new Guid("abe0296d-5394-4dff-8453-5440c28f4696");

        /// <summary>
        /// VS Package that provides this command, not null.
        /// </summary>
        private readonly Package package;
        private DTE2 dte2;

        /// <summary>
        /// Initializes a new instance of the <see cref="MacroCommand"/> class.
        /// Adds our command handlers for menu (commands must exist in the command table file)
        /// </summary>
        /// <param name="package">Owner package, not null.</param>
        private MacroCommand(Package package)
        {
            if (package == null)
            {
                throw new ArgumentNullException("package");
            }

            this.package = package;
            dte2 = (DTE2)this.ServiceProvider.GetService(typeof(DTE));

            OleMenuCommandService commandService = this.ServiceProvider.GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if (commandService != null)
            {
                for (int i = 0; i < 4; i++)
                {
                    var menuCommandID = new CommandID(CommandSet, CommandId + i);
                    var menuItem = new OleMenuCommand(this.MenuItemCallback, menuCommandID);
                    menuItem.BeforeQueryStatus += MenuItem_BeforeQueryStatus;
                    commandService.AddCommand(menuItem);
                }
            }
        }

        private void MenuItem_BeforeQueryStatus(object sender, EventArgs e)
        {
            OleMenuCommand menu = sender as OleMenuCommand;
            if (menu != null)
            {
                if (menu.CommandID.ID >= 0x3003)
                {
                    menu.Enabled = false;
                    menu.Visible = false;
                    return;
                }

                if (menu.CommandID.ID == 0x3001)
                {
                    menu.Text = "MakeAHref"; // 첫 번째 메뉴
                }

            }
        }

        /// <summary>
        /// Gets the instance of the command.
        /// </summary>
        public static MacroCommand Instance
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the service provider from the owner package.
        /// </summary>
        private IServiceProvider ServiceProvider
        {
            get
            {
                return this.package;
            }
        }

        /// <summary>
        /// Initializes the singleton instance of the command.
        /// </summary>
        /// <param name="package">Owner package, not null.</param>
        public static void Initialize(Package package)
        {
            Instance = new MacroCommand(package);
        }

        /// <summary>
        /// This function is the callback used to execute the command when the menu item is clicked.
        /// See the constructor to see how the menu item is associated with this function using
        /// OleMenuCommandService service and MenuCommand class.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event args.</param>
        private void MenuItemCallback(object sender, EventArgs e)
        {
            MenuCommand menu = sender as MenuCommand;

            if (menu != null)
            {
                RunCommand(menu.CommandID.ID);
            }
        }

        private void RunCommand(int id)
        {
            switch (id)
            {
                case 0x3001:
                    MakeAHref(this.dte2, this.package);
                    break;
            }
        }

        private void MakeAHref(EnvDTE80.DTE2 dte, Package package)
        {
            if (dte.ActiveDocument == null)
            {
                return;
            }

            TextSelection selection = dte.ActiveDocument.Selection as TextSelection;

            var str = "<a target='tab' href='" + DoHtmlEncode(selection.Text) + "'>" + selection.Text + "</a>";

            SetText(dte, selection, str);
        }

        public string DoHtmlEncode(string text)
        {
            text = text.Replace("&", "&amp;");
            text = text.Replace("<", "&lt;");
            text = text.Replace(">", "&gt;");

            return text;
        }

        public void SetText(EnvDTE80.DTE2 dte, TextSelection iSelection, string iText)
        {
            try
            {
                dte.UndoContext.Open("SetTextMakeAHref");
            }
            catch
            {
                dte.UndoContext.SetAborted();
                return;
            }

            var prop = dte.DTE.Properties["TextEditor", "PlainText"].Item("IndentStyle");
            var oldIndent = prop.Value;
            prop.Value = 0;

            iSelection.Text = iText;

            prop.Value = oldIndent;

            dte.UndoContext.Close();
        }
    }
}
